import 'package:ecommerce_basic/ecommerce_basic.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_inventory/src/repository/category.dart';

abstract class CategoryListEvent {}

class CategoryListRefreshEvent extends CategoryListEvent {}

abstract class CategoryListState {}

class CategoryListInitialState extends CategoryListState {}

class CategoryListLoadingState extends CategoryListState {}

class CategoryListLoadedState extends CategoryListState {
  final List<Category> categorys;
  CategoryListLoadedState(this.categorys);
}

class CategoryListErrorState extends CategoryListState {
  final String error;
  CategoryListErrorState(this.error);
}

class CategoryListBloc extends Bloc<CategoryListEvent, CategoryListState> {
  final CategoryRepository categoryRepository;
  CategoryListBloc(this.categoryRepository) : super(CategoryListInitialState());
  @override
  Stream<CategoryListState> mapEventToState(CategoryListEvent event) async* {
    if (event is CategoryListRefreshEvent) {
      yield CategoryListLoadingState();
      AppResponse<List<Category>> response = await categoryRepository.getList();
      if (response.isSuccess) {
        yield CategoryListLoadedState(response.data);
      } else {
        yield CategoryListErrorState(response.error);
      }
    }
  }
}
