import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_inventory/src/models/product.dart';
import 'package:ocg_inventory/src/repository/product/base.dart';

abstract class ProductCreateEvent {}

class ProductCreateStartEvent extends ProductCreateEvent {
  final Product product;
  ProductCreateStartEvent(this.product);
}

abstract class ProductCreateState {}

class ProductCreateIdleState extends ProductCreateState {}

class ProductCreateLoadingState extends ProductCreateState {}

class ProductCreateSuccessState extends ProductCreateState {
  final String id;
  ProductCreateSuccessState(this.id);
}

class ProductCreateErrorState extends ProductCreateState {
  final String error;
  ProductCreateErrorState(this.error);
}

class ProductCreateBloc extends Bloc<ProductCreateEvent, ProductCreateState> {
  final ProductRepository productRepository;
  ProductCreateBloc(this.productRepository) : super(ProductCreateIdleState());
  @override
  Stream<ProductCreateState> mapEventToState(ProductCreateEvent event) async* {
    if (event is ProductCreateStartEvent) {
      yield ProductCreateLoadingState();
      AppResponse<String> response =
          await productRepository.create(event.product);
      if (response.isSuccess) {
        yield ProductCreateSuccessState(response.data);
      } else {
        yield ProductCreateErrorState(response.error);
      }
    }
  }
}
