import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_inventory/src/models/product.dart';
import 'package:ocg_inventory/src/repository/product/base.dart';

abstract class ProductListEvent {}

class ProductListRefreshEvent extends ProductListEvent {}

class ProductListRefreshStoreEvent extends ProductListEvent {
  final String storeId;

  ProductListRefreshStoreEvent(this.storeId);
}

abstract class ProductListState {}

class ProductListInitialState extends ProductListState {}

class ProductListLoadingState extends ProductListState {}

class ProductListLoadedState extends ProductListState {
  final List<Product> products;

  ProductListLoadedState(this.products);
}

class ProductListErrorState extends ProductListState {
  final String error;

  ProductListErrorState(this.error);
}

class ProductListBloc extends Bloc<ProductListEvent, ProductListState> {
  final ProductRepository productRepository;
  ProductListBloc(this.productRepository) : super(ProductListInitialState());

  @override
  Stream<ProductListState> mapEventToState(ProductListEvent event) async* {
    if (event is ProductListRefreshEvent) {
      yield ProductListLoadingState();
      AppResponse<List<Product>> response = await productRepository.getList();
      if (response.isSuccess) {
        yield ProductListLoadedState(response.data);
      } else {
        yield ProductListErrorState(response.error);
      }
    }
    if (event is ProductListRefreshStoreEvent) {
      yield ProductListLoadingState();
      AppResponse<List<Product>> response =
          await productRepository.getProductByStore(event.storeId);
      if (response.isSuccess) {
        yield ProductListLoadedState(response.data);
      } else {
        yield ProductListErrorState(response.error);
      }
    }
  }
}
