import 'package:ecommerce_basic/ecommerce_basic.dart';
import 'package:json_annotation/json_annotation.dart';

part 'product.g.dart';

@JsonSerializable(explicitToJson: true)
class Product {
  final String id;
  final String storeId;
  final String name;
  final String description;
  final String thumbnail;
  @JsonKey(toJson: Product.toCategoryJson)
  final List<Category> categories;
  final Rating rating;
  final Price price;

  Product(
    this.id,
    this.storeId,
    this.name,
    this.description,
    this.thumbnail,
    this.categories,
    this.rating,
    this.price,
  );

  Product.named(
      {this.id,
      this.storeId,
      this.name,
      this.description,
      this.thumbnail,
      this.categories,
      this.rating,
      this.price});

  Product copyWith({
    String id,
    String storeId,
    String name,
    String description,
    String thumbnail,
    List<Category> categories,
    Rating rating,
    Price price,
  }) {
    return Product.named(
      id: id ?? this.id,
      name: name ?? this.name,
      categories: categories ?? this.categories,
      description: description ?? this.description,
      price: price ?? this.price,
      rating: rating ?? this.rating,
      storeId: storeId ?? this.storeId,
      thumbnail: thumbnail ?? this.thumbnail,
    );
  }

  Map<String, dynamic> toJson() => _$ProductToJson(this);

  factory Product.fromJson(json) => _$ProductFromJson(json);

  static toCategoryJson(List<Category> categories) {
    return categories.map((e) => e.id);
  }
}
