import 'package:ecommerce_basic/ecommerce_basic.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:uuid/uuid.dart';

abstract class CategoryRepository extends Crud<Category> {}

class CategoryTestRepository extends CategoryRepository {
  List<Category> _categories;

  CategoryTestRepository() : this._categories = [];

  @override
  Future<AppResponse<String>> create(Category object) async {
    if (object.name == null || object.name.isEmpty) {
      return AppResponse.named(error: "Name can not be empty");
    }
    object = object.copyWith(id: Uuid().v4());
    _categories.add(object);
    return AppResponse.named(data: object.id);
  }

  @override
  Future<AppResponse<bool>> delete(String id) async {
    _categories.removeWhere((element) => element.id == id);
    return AppResponse.named(data: true);
  }

  @override
  Future<AppResponse<List<Category>>> getList(
      {int page = 1, int count = 10}) async {
    return AppResponse.named(
        data: _categories.skip(count * page).take(count).toList());
  }

  @override
  Future<AppResponse<Category>> read(String id) async {
    int index = _categories.indexWhere((element) => element.id == id);
    if (index != -1) {
      return AppResponse.named(data: _categories[index]);
    }
    return AppResponse.named(error: "Category not exists");
  }

  @override
  Future<AppResponse<bool>> update(Category object) async {
    int index = _categories.indexWhere((element) => element.id == object.id);
    if (index != -1) {
      _categories[index] = object;
      return AppResponse.named(data: true);
    }
    return AppResponse.named(error: "Category not exists");
  }
}
