import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/foundation.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_inventory/src/models/product.dart';
import 'package:ocg_inventory/src/repository/product/base.dart';

class ProductFirebaseRepository extends ProductRepository {
  final IDHelper idHelper;
  final FirebaseFirestore _firestore;
  String get products => "products";
  CollectionReference get productCollection => _firestore.collection(products);
  ProductFirebaseRepository(
    this.idHelper,
  ) : _firestore = FirebaseFirestore.instance;
  Future<AppResponse<String>> create(Product product) async {
    try {
      if (InputHelper.isBlank(product.name)) {
        return AppResponse.named(error: "Name can not be empty");
      }
      if (InputHelper.isBlank(product.storeId)) {
        return AppResponse.named(error: "Store can not be empty");
      }
      String id = idHelper.generateId();
      product = product.copyWith(id: id);
      var json = product.toJson();
      await productCollection.doc(id).set(json);
      return AppResponse.named(data: id);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  Future<AppResponse<bool>> delete(String id) async {
    try {
      await productCollection.doc(id).delete();
      return AppResponse.named(data: true);
    } catch (e) {
      debugPrint("Error $e");
      return AppResponse.named(error: "$e");
    }
  }

  Future<AppResponse<List<Product>>> getList(
      {int page = 1, int count = 10}) async {
    try {
      var snapshot = await productCollection.get();
      if (snapshot.docs.length > 0) {
        var list = await Future.wait(snapshot.docs.map((e) async {
          return await _parseJson(e.data());
        }));
        list = list.where((element) => element != null).toList();
        return AppResponse.named(data: list);
      }
      return AppResponse.named(data: []);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  Future<AppResponse<Product>> read(String id) async {
    try {
      var snapshot = await productCollection.doc(id).get();
      if (snapshot.exists) {
        return AppResponse.named(data: await _parseJson(snapshot.data()));
      } else {
        return AppResponse.named(error: "Not exists");
      }
    } catch (e) {
      debugPrint("Error $e");
      return AppResponse.named(error: "$e");
    }
  }

  Future<Product> _parseJson(var json) async {
    return Product.fromJson(json);
  }

  Future<AppResponse<bool>> update(Product object) async {
    try {
      await _firestore.doc(object.id).update(object.toJson());
      return AppResponse.named(data: true);
    } catch (e) {
      return AppResponse.named(error: "Error $e");
    }
  }

  @override
  Future<AppResponse<List<Product>>> getProductByStore(String storeId,
      {int count, int page}) async {
    try {
      var snapshot =
          await productCollection.where("storeId", isEqualTo: storeId).get();
      if (snapshot.docs.length > 0) {
        var list = await Future.wait(snapshot.docs.map((e) async {
          return await _parseJson(e.data());
        }));
        list = list.where((element) => element != null).toList();
        return AppResponse.named(data: list);
      }
      return AppResponse.named(data: []);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }
}
